<?xml version="1.0" encoding="utf-8"?>

<xsl:stylesheet version="1.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:cf="http://tempuri.org/tvdatenDataSet.xsd"
                xmlns:ext="http://exslt.org/common"
                exclude-result-prefixes="cf ext">

  <xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes" />

  <!-- ************************************************************ -->
  <!-- Übergabeparameter -->
  <!-- ************************************************************ -->

  <xsl:param name = "imagesRootPath"/>
  <xsl:param name = "channelPostfix"/>
  <xsl:param name = "generationDateTime"/>
  <xsl:param name = "surroundCategoryChar"/>
  <xsl:param name = "creditsWriter"/>
  <xsl:param name = "creditsAdapter"/>
  <xsl:param name = "creditsProducer"/>
  <xsl:param name = "creditsComposer"/>
  <xsl:param name = "creditsEditor"/>
  <xsl:param name = "creditsPresenter"/>
  <xsl:param name = "creditsCommentator"/>
  <xsl:param name = "creditsGuest"/>
  <xsl:param name = "creditsDirector"/>
  <xsl:param name = "creditsNoUse"/>
  <xsl:param name = "convertFSK"/>
  <xsl:param name = "fskTo"/>
  <xsl:param name = "fsk0To"/>
  <xsl:param name = "fsk6To"/>
  <xsl:param name = "fsk12To"/>
  <xsl:param name = "fsk16To"/>
  <xsl:param name = "fsk18To"/>
  <!--<xsl:param name = "fsk0Icon"/>
  <xsl:param name = "fsk6Icon"/>
  <xsl:param name = "fsk12Icon"/>
  <xsl:param name = "fsk16Icon"/>
  <xsl:param name = "fsk18Icon"/>-->
  <xsl:param name = "withPictures"/>
  <xsl:param name = "macroXmltvSubTitle" />
  <xsl:param name = "macroXmltvTitle" />
  <xsl:param name = "macroXmltvDesc" />
  <xsl:param name = "macroXmltvCategory" />
  <xsl:param name = "macroFskMin"/>
  <xsl:param name = "macroFskFormat"/>
  <xsl:param name = "macroLandFormat"/>
  <xsl:param name = "macroJahrFormat"/>
  <xsl:param name = "macroInteressantFormat"/>
  <xsl:param name = "macroBewertungenFormat"/>
  <xsl:param name = "macroStaffelFormat"/>
  <xsl:param name = "macroFolgeFormat"/>
  <xsl:param name = "macroGenreFormat"/>
  <xsl:param name = "macroReplacePattern" />
  <xsl:param name = "xmltvStarBarTrue" />
  <xsl:param name = "xmltvStarBarFalse" />
  <xsl:param name = "maxStarRating" />
  <xsl:param name = "useEitGenre" />

  <xsl:variable name="smallcase" select="'abcdefghijklmnopqrstuvwxyz'" />
  <xsl:variable name="uppercase" select="'ABCDEFGHIJKLMNOPQRSTUVWXYZ'" />


  <!-- ************************************************************ -->
  <!-- Container -->
  <!-- ************************************************************ -->

  <xsl:template match="/">
    <tv generator-info-name="Dlx.Xmltv.KissCon">
      <xsl:apply-templates select="cf:tvdatenDataSet" />
    </tv>
  </xsl:template>

  <xsl:template match="cf:tvdatenDataSet">
    <xsl:apply-templates select="cf:Sender" />
    <xsl:apply-templates select="cf:TVDaten" />
  </xsl:template>

  <!-- ************************************************************ -->
  <!-- Sender -->
  <!-- ************************************************************ -->

  <xsl:template match="cf:Sender">
    <channel>

      <!-- id -->
      <xsl:apply-templates select="cf:SenderKennung" mode="channel"/>

      <!-- display-name -->
      <display-name>
        <xsl:call-template name="language"/>
        <xsl:value-of select="cf:Bezeichnung" />
      </display-name>

      <!-- icon -->
      <!--<xsl:apply-templates select="cf:Zeichen" />-->

    </channel>
  </xsl:template>

  <!-- ************************************************************ -->
  <!-- Sendungen -->
  <!-- ************************************************************ -->

  <xsl:template match="cf:TVDaten">
    <programme>

      <!-- ACHTUNG: Reihenfolge beachten, da sonst DTD-Validierung fehlschlägt! -->

      <!-- start -->
      <xsl:attribute name="start">
        <xsl:apply-templates select="cf:Beginn"/>
      </xsl:attribute>

      <!-- stop -->
      <xsl:attribute name="stop">
        <xsl:apply-templates select="cf:Ende"/>
      </xsl:attribute>

      <!-- channel -->
      <xsl:apply-templates select="cf:SenderKennung" mode="programme"/>

      <!-- title -->
      <title>
        <!--<xsl:call-template name="language"/>
        <xsl:call-template name="macro-replace">
          <xsl:with-param name="macroText" select="$macroXmltvTitle"/>
        </xsl:call-template>-->
        <xsl:call-template name="language"/>
        <xsl:choose>
          <xsl:when test="string-length(cf:Genre) > 0 and string-length(cf:Herstellungsjahr) > 0 and cf:Film = 'true'">
             <xsl:value-of select="concat(cf:Sendung, ' (', cf:Herstellungsjahr, ')')" />
          </xsl:when>
          <xsl:otherwise>
            <xsl:call-template name="macro-replace">
              <xsl:with-param name="macroText" select="$macroXmltvTitle"/>
            </xsl:call-template>
          </xsl:otherwise>
        </xsl:choose>
      </title>

      <!-- sub-title -->
      <!--<xsl:if test="string-length(cf:Originaltitel) > 0">-->
      <sub-title>
        <xsl:call-template name="language"/>
        <xsl:call-template name="macro-replace">
          <xsl:with-param name="macroText" select="$macroXmltvSubTitle"/>
        </xsl:call-template>
      </sub-title>
      <!--</xsl:if>-->

      <!-- desc -->
      <desc>
        <xsl:call-template name="language"/>
        <xsl:call-template name="macro-replace">
          <xsl:with-param name="macroText" select="$macroXmltvDesc"/>
        </xsl:call-template>
      </desc>

      <!-- credits -->
      <xsl:if test="string-length(cf:Regie) > 0 or string-length(cf:Darsteller) > 0">
        <credits>
          <xsl:if test="string-length(cf:Regie) > 0">
            <director>
              <xsl:value-of select="cf:Regie" />
            </director>
          </xsl:if>

          <xsl:if test="string-length(cf:Darsteller) > 0">
            <xsl:apply-templates select="cf:Darsteller"/>
          </xsl:if>

        </credits>
      </xsl:if>

      <!-- date -->
      <xsl:if test="string-length(cf:Herstellungsjahr) > 0">
        <date>
          <xsl:value-of select="cf:Herstellungsjahr" />
        </date>
      </xsl:if>

      <!-- category -->
      <xsl:choose>
        <xsl:when test="$useEitGenre = 0">

          <xsl:if test="string-length(cf:Genre) > 0">
            <category>
              <xsl:call-template name="language"/>
              <xsl:choose>

                <xsl:when test="contains(translate(cf:Keywords,$smallcase,$uppercase),'REIHE')">
                  <xsl:value-of select="concat($surroundCategoryChar, cf:Genre, ' (Serie)', $surroundCategoryChar)" />
                </xsl:when>

                <!--<xsl:when test="contains(translate(cf:Keywords,$smallcase,$uppercase),'FILM')">
                <xsl:value-of select="concat($surroundCategoryChar, cf:Genre, ' (Film)', $surroundCategoryChar)" />
              </xsl:when>-->

                <xsl:when test="contains(translate(cf:Keywords,$smallcase,$uppercase),'SERIE')">
                  <xsl:value-of select="concat($surroundCategoryChar, cf:Genre, ' (Serie)', $surroundCategoryChar)" />
                </xsl:when>

                <xsl:when test="cf:Film = 'true'">
                  <xsl:value-of select="concat($surroundCategoryChar, cf:Genre, ' (Film)', $surroundCategoryChar)" />
                </xsl:when>

                <xsl:otherwise>
                  <!--<xsl:value-of select="concat(cf:Genre, ', ', cf:Kategorietext)" />-->
                  <xsl:value-of select="concat($surroundCategoryChar, cf:Genre, $surroundCategoryChar)" />
                </xsl:otherwise>

              </xsl:choose>
            </category>
          </xsl:if>

        </xsl:when>
        <xsl:otherwise>

          <category lang="en">
            <xsl:value-of select="cf:KodiGenre" />
          </category>

        </xsl:otherwise>
      </xsl:choose>


      <!--
      <xsl:variable name="category">
        <xsl:call-template name="macro-replace">
          <xsl:with-param name="macroText" select="$macroXmltvCategory"/>
        </xsl:call-template>
      </xsl:variable>
      <xsl:if test="string-length($category) > 0">

        <xsl:variable name="categorySplit">
          <xsl:call-template name="split">
            <xsl:with-param name="text2Split" select="translate($category,',',';')" />
          </xsl:call-template>
        </xsl:variable>

        <xsl:for-each select="ext:node-set($categorySplit)/*">

          <category>
            <xsl:call-template name="language"/>

            <xsl:choose>

              <xsl:when test="contains(translate(cf:Keywords,$smallcase,$uppercase),'SERIE')">
                <xsl:value-of select="concat($surroundCategoryChar, ., ' (Serie)', $surroundCategoryChar)" />
              </xsl:when>

              <xsl:when test="contains(translate(cf:Keywords,$smallcase,$uppercase),'REIHE')">
                <xsl:value-of select="concat($surroundCategoryChar, ., ' (Serie)', $surroundCategoryChar)" />
              </xsl:when>

              <xsl:when test="cf:Film = 'true'">
                <xsl:value-of select="concat($surroundCategoryChar, ., ' (Film)', $surroundCategoryChar)" />
              </xsl:when>

              <xsl:otherwise>
                <xsl:value-of select="concat($surroundCategoryChar, ., $surroundCategoryChar)" />
              </xsl:otherwise>

            </xsl:choose>

          </category>
        </xsl:for-each>
      </xsl:if>
      -->

      <!-- length -->
      <length units="minutes">
        <xsl:value-of select="substring-before(cf:Dauer,' ')" />
      </length>

      <!-- icon -->
      <!--<xsl:if test="cf:BilddateiHeruntergeladen = 'true'">-->
      <xsl:if test="boolean($withPictures)">
        <xsl:if test="string-length(cf:Bilddateiname) > 0">
          <xsl:apply-templates select="cf:Bilddateiname"/>
        </xsl:if>
      </xsl:if>
      <!--</xsl:if>-->

      <!-- country -->
      <xsl:if test="string-length(cf:Herstellungsland) > 0">
        <country>
          <xsl:call-template name="language"/>
          <xsl:value-of select="cf:Herstellungsland" />
        </country>
      </xsl:if>

      <!-- episode-num -->
      <xsl:call-template name="episode-num"/>
      <!--<xsl:variable name="episodeString" select="substring-before(cf:Beschreibung,':')" />
      <xsl:if test="string-length($episodeString) > 0">
        <xsl:if test="contains($episodeString, 'Staffel, Folge')">
          <episode-num>
            <xsl:variable name="season" select="number(substring-before($episodeString,'. Staffel'))-1" />
            <xsl:variable name="episode" select="number(substring-after($episodeString,'Folge '))-1" />
            <xsl:value-of select="concat($season,'.',$episode,'.')"/>
          </episode-num>
        </xsl:if>
      </xsl:if>-->

      <!-- video -->
      <video>
        <present>yes</present>

        <colour>
          <xsl:if test="cf:schwarzweis = 'false'">yes</xsl:if>
        </colour>

        <xsl:if test="cf:F16zu9 = 'true'">
          <aspect>16:9</aspect>
        </xsl:if>

        <xsl:variable name="shortDesc" select="cf:KurzBeschreibung"/>
        <xsl:if test="string-length($shortDesc) >= 4 and substring($shortDesc, 1, 4) = 'HDTV'">
          <quality>HDTV</quality>
        </xsl:if>

      </video>

      <!-- audio -->
      <audio>
        <present>yes</present>

        <xsl:choose>
          <xsl:when test="cf:DolbyDigital = 'true'">
            <stereo>dolby digital</stereo>
          </xsl:when>
          <xsl:when test="cf:Dolby = 'true'">
            <stereo>dolby</stereo>
          </xsl:when>
          <xsl:when test="cf:dolbysuround = 'true'">
            <stereo>surround</stereo>
          </xsl:when>
          <xsl:when test="cf:zweikanalton = 'true'">
            <stereo>bilingual</stereo>
          </xsl:when>
          <xsl:when test="cf:stereo = 'true'">
            <stereo>stereo</stereo>
          </xsl:when>
          <xsl:otherwise>
            <stereo>mono</stereo>
          </xsl:otherwise>
        </xsl:choose>
      </audio>

      <!-- previously-shown -->
      <xsl:if test="cf:Wiederholung = 'true'">
        <previously-shown>
          <!-- deaktiviert, da rechenintensiv und in WMC scheinbar nicht benötigt -->
          <!--<xsl:apply-templates select="cf:InhaltID"/>-->
        </previously-shown>
      </xsl:if>

      <!-- subtitles -->
      <xsl:if test="cf:untertitel = 'true'">
        <subtitles type="teletext" />
      </xsl:if>

      <!-- rating -->
      <!-- FSK -->
      <!--<xsl:if test="not($convertFSK)" >-->
      <rating system="FSK">
        <value>
          <xsl:value-of select="cf:FSK" />
        </value>
        <!--<icon>
          <xsl:attribute name="src">
            <xsl:choose>
              <xsl:when test="cf:FSK = '0'">
                <xsl:value-of select="$fsk0Icon"/>
              </xsl:when>
              <xsl:when test="cf:FSK = '6'">
                <xsl:value-of select="$fsk6Icon"/>
              </xsl:when>
              <xsl:when test="cf:FSK = '12'">
                <xsl:value-of select="$fsk12Icon"/>
              </xsl:when>
              <xsl:when test="cf:FSK = '16'">
                <xsl:value-of select="$fsk16Icon"/>
              </xsl:when>
              <xsl:when test="cf:FSK = '18'">
                <xsl:value-of select="$fsk18Icon"/>
              </xsl:when>
            </xsl:choose>
          </xsl:attribute>
        </icon>-->
      </rating>
      <!--</xsl:if>-->

      <!-- MPAA -->
      <xsl:if test="$convertFSK" >
        <rating>
          <xsl:attribute name="system">
            <xsl:value-of select="$fskTo"/>
          </xsl:attribute>
          <xsl:choose>
            <!-- FSK18 -->
            <xsl:when test="number(cf:FSK) &gt; 16">
              <value>
                <xsl:value-of select="$fsk18To"/>
              </value>
              <!--<icon>
                <xsl:attribute name="src">
                  <xsl:value-of select="$fsk18Icon"/>
                </xsl:attribute>
              </icon>-->
            </xsl:when>
            <!-- FSK16 -->
            <xsl:when test="number(cf:FSK) &gt; 12">
              <value>
                <xsl:value-of select="$fsk16To"/>
              </value>
              <!--<icon>
                <xsl:attribute name="src">
                  <xsl:value-of select="$fsk16Icon"/>
                </xsl:attribute>
              </icon>-->
            </xsl:when>
            <!-- FSK12 -->
            <xsl:when test="number(cf:FSK) &gt; 6">
              <value>
                <xsl:value-of select="$fsk12To"/>
              </value>
              <!--<icon>
                <xsl:attribute name="src">
                  <xsl:value-of select="$fsk12Icon"/>
                </xsl:attribute>
              </icon>-->
            </xsl:when>
            <!-- FSK6 -->
            <xsl:when test="number(cf:FSK) &gt; 0">
              <value>
                <xsl:value-of select="$fsk6To"/>
              </value>
              <!--<icon>
                <xsl:attribute name="src">
                  <xsl:value-of select="$fsk6Icon"/>
                </xsl:attribute>
              </icon>-->
            </xsl:when>
            <!-- FSK0 -->
            <xsl:otherwise>
              <value>
                <xsl:value-of select="$fsk0To"/>
              </value>
              <!--<icon>
                <xsl:attribute name="src">
                  <xsl:value-of select="$fsk0Icon"/>
                </xsl:attribute>
              </icon>-->
            </xsl:otherwise>
          </xsl:choose>
        </rating>
      </xsl:if>

      <!-- star-rating -->
      <xsl:if test="number(cf:Interessant) > 0 and number(cf:Interessant) &lt; 6">
        <star-rating>
          <value>
            <xsl:choose>
              <xsl:when test="number(cf:Interessant) = 1">
                <xsl:value-of select="'1/4'" />
              </xsl:when>
              <xsl:when test="number(cf:Interessant) = 2">
                <xsl:value-of select="'2/4'" />
              </xsl:when>
              <xsl:when test="number(cf:Interessant) = 3">
                <xsl:value-of select="'3/4'" />
              </xsl:when>
              <xsl:when test="number(cf:Interessant) = 4">
                <xsl:value-of select="'3/4'" />
              </xsl:when>
              <xsl:when test="number(cf:Interessant) = 5">
                <xsl:value-of select="'4/4'" />
              </xsl:when>
            </xsl:choose>
            <!--<xsl:value-of select="concat(number(cf:Interessant) - 1,'/4')" />-->
          </value>
        </star-rating>
      </xsl:if>
      <xsl:if test="number(cf:Interessant) &lt; 0 and number(cf:Interessant) >= -100">
        <star-rating>
          <value>
            <xsl:value-of select="concat(number(substring-after(cf:Interessant,'-')) div 10,'/',$maxStarRating)" />
          </value>
        </star-rating>
      </xsl:if>

      <!-- review -->
      <xsl:if test="string-length(cf:Kurzkritik) > 0">
        <review type="text">
          <xsl:call-template name="language"/>
          <xsl:value-of select="concat(cf:Bewertungen, ' - ', cf:Kurzkritik)" />
        </review>
      </xsl:if>

    </programme>
  </xsl:template>

  <!-- ************************************************************ -->
  <!-- TEMPLATES und Funktionen -->
  <!-- ************************************************************ -->

  <!-- Sucht Esrtausstrahlung zu einer Wiederholung -->
  <!-- ACHTUNG: wird momentan nicht verwendet, da rechenintensiv und WMC Wiederholungen allein durch das Flag erkennt -->
  <xsl:template match="cf:InhaltID">
    <xsl:attribute name="start">
      <xsl:variable name="showId" select="." />
      <!-- Version 1: einfacher Ansatz, Suche über alles -->
      <!--<xsl:apply-templates select="/cf:tvdatenDataSet/cf:TVDaten[cf:InhaltID = $showId][1]/cf:Beginn"/>-->
      <!-- Version 2: optimiert, da "nur" alle Knoten vor der Sendung durchsucht werden -->
      <xsl:apply-templates select="../preceding-sibling::*[cf:InhaltID = $showId][1]/cf:Beginn"/>
      <!-- Version 3: Suche ab Knoten rückwärts, bis Sendung gefunden und brich dann ab -->
      <!-- ToDo: Template bauen das rekursiv die Geschwister rückwärts durchsucht, bis es den passenden Eintrag gefunden hat und dann abbricht -->
      <!-- Version 4:-->
      <!-- ToDo: DataSet nach 'InhaltID' sortieren => Zwilling (wenn vorhanden) ist Knoten vor dem aktuellem -->
    </xsl:attribute>
  </xsl:template>

  <!-- Serieninformation -->
  <xsl:template name="episode-num">
    <xsl:variable name="season">
      <xsl:call-template name="season" />
    </xsl:variable>
    <xsl:variable name="episode">
      <xsl:call-template name="episode" />
    </xsl:variable>
    <xsl:if test="string-length($season) > 0 and string-length($episode) > 0">
      <episode-num system="xmltv_ns">
        <xsl:value-of select="concat($season - 1,'.',$episode - 1,'.')"/>
      </episode-num>
    </xsl:if>
  </xsl:template>

  <!-- Staffel einer Serie -->
  <xsl:template name="season">
    <!--<xsl:variable name="episodeString" select="substring-before(cf:Beschreibung,':')" />
    <xsl:if test="string-length($episodeString) > 0 and contains($episodeString, 'Staffel, Folge')">
      <xsl:value-of select="number(substring-before($episodeString,'. Staffel'))"/>
    </xsl:if>-->
    <xsl:if test="cf:HatSerienInfo = 'true'">
      <xsl:value-of select="cf:Staffel"/>
    </xsl:if>
  </xsl:template>

  <!-- Folge einer Serie -->
  <xsl:template name="episode">
    <!--<xsl:variable name="episodeString" select="substring-before(cf:Beschreibung,':')" />
    <xsl:if test="string-length($episodeString) > 0 and contains($episodeString, 'Staffel, Folge')">
      <xsl:value-of select="number(substring-after($episodeString,'Folge '))"/>
    </xsl:if>-->
    <xsl:if test="cf:HatSerienInfo = 'true'">
      <xsl:value-of select="cf:Folge"/>
    </xsl:if>
  </xsl:template>

  <!-- Fügt die Standardsprache als Attribute hinzu -->
  <xsl:template name="language">
    <xsl:attribute name="lang">de</xsl:attribute>
  </xsl:template>

  <!-- Icon für Sendung und Sender (Unterscheidung über 'subDirectory') -->
  <xsl:template name="icon">
    <!--<xsl:param name="subDirectory" />-->
    <xsl:param name="fileName" />
    <icon>
      <xsl:attribute name="src">
        <xsl:choose>
          <xsl:when test="substring($fileName,1,4) = 'http'">
            <xsl:value-of select="$fileName"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:variable name="urlSeperator">
              <xsl:choose>
                <xsl:when test="substring($imagesRootPath,1,4) = 'file'">
                  <xsl:value-of select="'\'"/>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:value-of select="'/'"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:variable>
            <xsl:value-of select="concat($imagesRootPath, $urlSeperator, $fileName)" />
            <!--<xsl:value-of select="concat($imagesRootPath, $urlSeperator, $subDirectory, $urlSeperator, $fileName)" />-->
          </xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
    </icon>
  </xsl:template>

  <!-- Icon für Sendung -->
  <xsl:template match="cf:Bilddateiname">
    <xsl:call-template name="icon">
      <!--<xsl:with-param name="subDirectory" select="'Hyperlinks'" />-->
      <xsl:with-param name="fileName" select="." />
    </xsl:call-template>
  </xsl:template>

  <!-- Icon für Sender -->
  <xsl:template match="cf:Zeichen">
    <xsl:call-template name="icon">
      <xsl:with-param name="subDirectory" select="'gifs'" />
      <xsl:with-param name="fileName" select="." />
    </xsl:call-template>
  </xsl:template>


  <!-- Sendernamen um eigenes Kürzel erweitern -->
  <xsl:template match="cf:SenderKennung">
    <xsl:param name="attributeName" />
    <xsl:attribute name="{$attributeName}">
      <xsl:value-of select="concat(., $channelPostfix)"/>
    </xsl:attribute>
  </xsl:template>

  <!-- Sendername als Attribut für Sendung (Attributname: channel) -->
  <xsl:template match="cf:SenderKennung" mode="programme">
    <xsl:apply-templates select="." >
      <xsl:with-param name="attributeName" select="'channel'"/>
    </xsl:apply-templates>
  </xsl:template>

  <!-- Sendername als Attribut für Sender (Attributname: id) -->
  <xsl:template match="cf:SenderKennung" mode="channel">
    <xsl:apply-templates select="." >
      <xsl:with-param name="attributeName" select="'id'"/>
    </xsl:apply-templates>
  </xsl:template>


  <!-- Zeitformat von clickfinder in XMLTV-Zeitformat konvertieren  -->
  <xsl:template match="cf:Beginn|cf:Ende">
    <!--<xsl:variable name="datetime" select="." />-->
    <xsl:value-of
             select="concat(
                      substring(., 1, 4),
                      substring(., 6, 2),
                      substring(., 9, 2),
                      substring(., 12, 2),
                      substring(., 15, 2),
                      substring(., 18, 2),
                      ' ',
                      substring(., 20, 3),
                      substring(., 24, 2)
                      )" />
  </xsl:template>


  <!-- Template um Darstellerstring aufzusplitten -->
  <xsl:template match="cf:Darsteller">
    <xsl:variable name="darstellerSplit">
      <xsl:call-template name="split"/>
    </xsl:variable>

    <xsl:variable name="directors"/>
    <xsl:variable name="writers"/>
    <xsl:variable name="adapters"/>
    <xsl:variable name="producers"/>
    <xsl:variable name="composers"/>
    <xsl:variable name="editors"/>
    <xsl:variable name="presenters"/>
    <xsl:variable name="commentators"/>
    <xsl:variable name="guests"/>

    <xsl:for-each select="ext:node-set($darstellerSplit)/*">
      <xsl:variable name="role" select="substring-before(substring-after(.,'('),')')"/>
      <xsl:variable name="roleUp" select="concat(',',translate($role,$smallcase,$uppercase),',')"/>
      <xsl:if test="string-length($role) > 0">
        <xsl:choose>
          <xsl:when test="contains(concat(',',translate($creditsDirector,$smallcase,$uppercase),','), $roleUp)">
            <!--<director>
              <xsl:value-of select="." />
            </director>-->
          </xsl:when>
          <xsl:when test="contains(concat(',',translate($creditsWriter,$smallcase,$uppercase),','), $roleUp)">
            <!--<writer>
              <xsl:value-of select="." />
            </writer>-->
          </xsl:when>
          <xsl:when test="contains(concat(',',translate($creditsAdapter,$smallcase,$uppercase),','), $roleUp)">
            <!--<adapter>
              <xsl:value-of select="." />
            </adapter>-->
          </xsl:when>
          <xsl:when test="contains(concat(',',translate($creditsProducer,$smallcase,$uppercase),','), $roleUp)">
            <!--<producer>
              <xsl:value-of select="." />
            </producer>-->
          </xsl:when>
          <xsl:when test="contains(concat(',',translate($creditsComposer,$smallcase,$uppercase),','), $roleUp)">
            <!--<composer>
              <xsl:value-of select="." />
            </composer>-->
          </xsl:when>
          <xsl:when test="contains(concat(',',translate($creditsEditor,$smallcase,$uppercase),','), $roleUp)">
            <!--<editor>
              <xsl:value-of select="." />
            </editor>-->
          </xsl:when>
          <xsl:when test="contains(concat(',',translate($creditsPresenter,$smallcase,$uppercase),','), $roleUp)">
            <!--<presenter>
              <xsl:value-of select="." />
            </presenter>-->
          </xsl:when>
          <xsl:when test="contains(concat(',',translate($creditsCommentator,$smallcase,$uppercase),','), $roleUp)">
            <!--<commentator>
              <xsl:value-of select="." />
            </commentator>-->
          </xsl:when>
          <!--<xsl:when test="contains(concat(',',translate($creditsGuest,$smallcase,$uppercase),','), $roleUp)">
            <guest>
              <xsl:value-of select="." />
            </guest>
          </xsl:when>-->
          <xsl:when test="contains(translate($creditsNoUse,$smallcase,$uppercase), $roleUp)"/>
          <xsl:otherwise>
            <actor>
              <xsl:attribute name="role">
                <xsl:value-of select="$role"/>
              </xsl:attribute>
              <xsl:value-of select="." />
            </actor>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:if>
    </xsl:for-each>
  </xsl:template>

  <!-- SPLIT-Funktion -->
  <xsl:template name="split">
    <xsl:param name="text2Split" select="." />
    <xsl:if test="string-length($text2Split) >0">

      <!-- ersten Teilstring abspalten und in <item> speichern -->
      <item>
        <xsl:value-of select="substring-before(concat($text2Split, ';'), ';')" />
      </item>

      <!-- Funktion rekursiv aufrufen um restliche Teilstrings abzuspalten -->
      <xsl:call-template name="split">
        <xsl:with-param name="text2Split" select="substring-after($text2Split, ';')" />
      </xsl:call-template>

    </xsl:if>
  </xsl:template>

  <!-- STAR-BAR-Funktion -->
  <xsl:template name="star-bar">
    <xsl:param name="max"/>
    <xsl:param name="value"/>
    <xsl:call-template name="string-pad">
      <xsl:with-param name="char" select="$xmltvStarBarTrue"/>
      <xsl:with-param name="count" select="$value"/>
    </xsl:call-template>
    <xsl:call-template name="string-pad">
      <xsl:with-param name="char" select="$xmltvStarBarFalse"/>
      <xsl:with-param name="count" select="$max - $value"/>
    </xsl:call-template>
  </xsl:template>

  <!-- PAD-Funktion -->
  <xsl:template name="string-pad">
    <xsl:param name="char" />
    <xsl:param name="count" />
    <xsl:choose>
      <xsl:when test="$count > 0">
        <xsl:value-of select="$char" />
        <xsl:call-template name="string-pad">
          <xsl:with-param name="char" select="$char" />
          <xsl:with-param name="count" select="$count - 1" />
        </xsl:call-template>
      </xsl:when>
    </xsl:choose>
  </xsl:template>

  <!-- REPLACE-Funktion -->
  <xsl:template name="string-replace">
    <xsl:param name="text" />
    <xsl:param name="replace" />
    <xsl:param name="by" />
    <xsl:choose>
      <xsl:when test="contains($text, $replace)">
        <xsl:value-of select="substring-before($text, $replace)" />
        <xsl:value-of select="$by" />
        <xsl:call-template name="string-replace">
          <xsl:with-param name="text" select="substring-after($text, $replace)" />
          <xsl:with-param name="replace" select="$replace" />
          <xsl:with-param name="by" select="$by" />
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$text" />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- ************************************************************ -->
  <!-- MAKRO-Engine -->
  <!-- ************************************************************ -->

  <!-- MAKRO-Replace-Funktion -->
  <xsl:template name="macro-replace">
    <xsl:param name="stackCount" select="0"/>
    <xsl:param name="macroText" />
    <xsl:choose>
      <xsl:when test="$stackCount &lt; 50 and contains($macroText, '{') and contains(substring-after($macroText,'{'), '}')">
        <xsl:value-of select="substring-before($macroText, '{')" />
        <xsl:call-template name="macro-replace">
          <xsl:with-param name="stackCount" select="$stackCount + 1"/>
          <xsl:with-param name="macroText" >
            <xsl:call-template name="makro-choose">
              <xsl:with-param name="macroType" select="substring-after(substring-before($macroText,'}'),'{')" />
            </xsl:call-template>
            <xsl:value-of select="substring-after($macroText, '}')"/>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$macroText" />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- MAKRO-Auswahl -->
  <xsl:template name="makro-choose">
    <xsl:param name="macroType"/>
    <xsl:variable name="smallMacroType"  select="translate($macroType,$uppercase,$smallcase)"/>
    <xsl:choose>
      <xsl:when test="$smallMacroType = 'if_film'">
        <xsl:call-template name="macro-If_Film" />
      </xsl:when>      
      <xsl:when test="$smallMacroType = 'n'">
        <xsl:call-template name="macro-N" />
      </xsl:when>
      <xsl:when test="$smallMacroType = 'bewertungen'">
        <xsl:call-template name="macro-Bewertungen" />
      </xsl:when>
      <xsl:when test="$smallMacroType = 'bewertungenmin'">
        <xsl:call-template name="macro-BewertungenMin" />
      </xsl:when>
      <xsl:when test="$smallMacroType = 'fsk'">
        <xsl:call-template name="macro-FSK" />
      </xsl:when>
      <xsl:when test="$smallMacroType = 'beschreibung'">
        <xsl:call-template name="macro-Beschreibung" />
      </xsl:when>
      <xsl:when test="$smallMacroType = 'keywords'">
        <xsl:call-template name="macro-Keywords" />
      </xsl:when>
      <xsl:when test="$smallMacroType = 'genre'">
        <xsl:call-template name="macro-Genre" />
      </xsl:when>
      <xsl:when test="$smallMacroType = 'kategorietext'">
        <xsl:call-template name="macro-Kategorietext" />
      </xsl:when>
      <xsl:when test="$smallMacroType = 'darsteller'">
        <xsl:call-template name="macro-Darsteller" />
      </xsl:when>
      <xsl:when test="$smallMacroType = 'kurzkritik'">
        <xsl:call-template name="macro-Kurzkritik" />
      </xsl:when>
      <xsl:when test="$smallMacroType = 'kurzbeschreibung'">
        <xsl:call-template name="macro-KurzBeschreibung" />
      </xsl:when>
      <xsl:when test="$smallMacroType = 'originaltitel'">
        <xsl:call-template name="macro-Originaltitel" />
      </xsl:when>
      <xsl:when test="$smallMacroType = 'sendung'">
        <xsl:call-template name="macro-Sendung" />
      </xsl:when>
      <xsl:when test="$smallMacroType = 'interessant'">
        <xsl:call-template name="macro-Interessant" />
      </xsl:when>
      <xsl:when test="$smallMacroType = 'staffel'">
        <xsl:call-template name="macro-Staffel" />
      </xsl:when>
      <xsl:when test="$smallMacroType = 'folge'">
        <xsl:call-template name="macro-Folge" />
      </xsl:when>
      <xsl:when test="$smallMacroType = 'jahr'">
        <xsl:call-template name="macro-Jahr" />
      </xsl:when>
      <xsl:when test="$smallMacroType = 'land'">
        <xsl:call-template name="macro-Land" />
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="concat('[unbekanntes Makro = ',$macroType,']')"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- MAKRO-Jahr -->
  <xsl:template name="macro-Jahr">
    <xsl:if test="string-length(cf:Herstellungsjahr) > 0">
      <xsl:call-template name="string-replace">
        <xsl:with-param name="text" select="$macroJahrFormat"/>
        <xsl:with-param name="replace" select="$macroReplacePattern"/>
        <xsl:with-param name="by" select="cf:Herstellungsjahr"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>

  <!-- MAKRO-Land -->
  <xsl:template name="macro-Land">
    <xsl:if test="string-length(cf:Herstellungsland) > 0">
      <xsl:call-template name="string-replace">
        <xsl:with-param name="text" select="$macroLandFormat"/>
        <xsl:with-param name="replace" select="$macroReplacePattern"/>
        <xsl:with-param name="by" select="cf:Herstellungsland"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>

  <!-- MAKRO-Staffel -->
  <xsl:template name="macro-Staffel">
    <xsl:variable name="season">
      <xsl:call-template name="season"/>
    </xsl:variable>
    <xsl:if test="string-length($season) > 0">
      <xsl:call-template name="string-replace">
        <xsl:with-param name="text" select="$macroStaffelFormat"/>
        <xsl:with-param name="replace" select="$macroReplacePattern"/>
        <xsl:with-param name="by" select="$season"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>

  <!-- MAKRO-Folge -->
  <xsl:template name="macro-Folge">
    <xsl:variable name="episode">
      <xsl:call-template name="episode"/>
    </xsl:variable>
    <xsl:if test="string-length($episode) > 0">
      <xsl:call-template name="string-replace">
        <xsl:with-param name="text" select="$macroFolgeFormat"/>
        <xsl:with-param name="replace" select="$macroReplacePattern"/>
        <xsl:with-param name="by" select="$episode"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>

  <!-- MAKRO-FSK -->
  <xsl:template name="macro-FSK">
    <xsl:if test="number(cf:FSK) >= $macroFskMin">
      <xsl:call-template name="string-replace">
        <xsl:with-param name="text" select="$macroFskFormat"/>
        <xsl:with-param name="replace" select="$macroReplacePattern"/>
        <xsl:with-param name="by" select="cf:FSK"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>

  <!-- MAKRO-Interessant -->
  <xsl:template name="macro-Interessant">
    <xsl:if test="number(cf:Interessant) > 0 and number(cf:Interessant) &lt; 6">
      <xsl:call-template name="string-replace">
        <xsl:with-param name="text" select="$macroInteressantFormat"/>
        <xsl:with-param name="replace" select="$macroReplacePattern"/>
        <xsl:with-param name="by">
          <xsl:call-template name="star-bar">
            <xsl:with-param name="max" select="4"/>
            <xsl:with-param name="value">
              <xsl:choose>
                <xsl:when test="number(cf:Interessant) = 1">
                  <xsl:value-of select="1" />
                </xsl:when>
                <xsl:when test="number(cf:Interessant) = 2">
                  <xsl:value-of select="2" />
                </xsl:when>
                <xsl:when test="number(cf:Interessant) = 3">
                  <xsl:value-of select="3" />
                </xsl:when>
                <xsl:when test="number(cf:Interessant) = 4">
                  <xsl:value-of select="3" />
                </xsl:when>
                <xsl:when test="number(cf:Interessant) = 5">
                  <xsl:value-of select="4" />
                </xsl:when>
              </xsl:choose>
            </xsl:with-param>
          </xsl:call-template>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:if>
    <xsl:if test="number(cf:Interessant) &lt; 0 and number(cf:Interessant) >= -100">
      <xsl:value-of select="'['"/>
      <xsl:call-template name="star-bar">
        <xsl:with-param name="max" select="10"/>
        <xsl:with-param name="value" select="round(number(substring-after(cf:Interessant,'-')) div 10)" />
      </xsl:call-template>
      <xsl:value-of select="']'"/>
    </xsl:if>
  </xsl:template>

  <!-- MAKRO-Newline -->
  <xsl:template name="macro-N">
    <xsl:value-of select="' &#13;&#10;'"/>
  </xsl:template>
  
  <!-- MAKRO-Newline -->
  <xsl:template name="macro-If_Film">
    <xsl:value-of select="' &#13;&#10;'"/>
  </xsl:template>

  <!-- MAKRO-Bewertungen -->
  <xsl:template name="macro-Bewertungen">
    <xsl:if test="string-length(cf:Bewertungen) > 0">
      <xsl:value-of select="cf:Bewertungen"/>
    </xsl:if>
  </xsl:template>

  <!-- MAKRO-BewertungenMin -->
  <xsl:template name="macro-BewertungenMin">
    <xsl:if test="string-length(cf:Bewertungen) > 0">
      <xsl:call-template name="string-replace">
        <xsl:with-param name="text" select="$macroBewertungenFormat"/>
        <xsl:with-param name="replace" select="$macroReplacePattern"/>
        <xsl:with-param name="by">
          <xsl:variable name="bewertungenSplit">
            <xsl:call-template name="split">
              <xsl:with-param name="text2Split" select="cf:Bewertungen"/>
            </xsl:call-template>
          </xsl:variable>
          <xsl:for-each select="ext:node-set($bewertungenSplit)/*">
            <xsl:variable name="bewertungValue" select="number(substring-after(.,'='))"/>
            <xsl:if test="$bewertungValue > 0">
              <xsl:value-of select="concat('[',substring-before(.,'='),' ')"/>
              <xsl:call-template name="star-bar">
                <xsl:with-param name="max" select="3"/>
                <xsl:with-param name="value" select="$bewertungValue"/>
              </xsl:call-template>
              <xsl:value-of select="']'"/>
            </xsl:if>
          </xsl:for-each>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>

  <!-- MAKRO-KurzBeschreibung -->
  <xsl:template name="macro-KurzBeschreibung">
    <xsl:if test="string-length(cf:KurzBeschreibung) > 4">
      <xsl:value-of select="cf:KurzBeschreibung"/>
    </xsl:if>
  </xsl:template>

  <!-- MAKRO-Originaltitel -->
  <xsl:template name="macro-Originaltitel">
    <xsl:if test="string-length(cf:Originaltitel) > 0">
      <xsl:value-of select="cf:Originaltitel" />
    </xsl:if>
  </xsl:template>

  <!-- MAKRO-Sendung -->
  <xsl:template name="macro-Sendung">
    <xsl:if test="string-length(cf:Sendung) > 0">
      <xsl:value-of select="cf:Sendung" />
    </xsl:if>
  </xsl:template>

  <!-- MAKRO-Keywords -->
  <xsl:template name="macro-Keywords">
    <xsl:value-of select="cf:Keywords"/>
  </xsl:template>

  <!-- MAKRO-Kurzkritik -->
  <xsl:template name="macro-Kurzkritik">
    <xsl:value-of select="cf:Kurzkritik"/>
  </xsl:template>

  <!-- MAKRO-Genre -->
  <xsl:template name="macro-Genre">
    <xsl:call-template name="string-replace">
      <xsl:with-param name="text" select="$macroGenreFormat"/>
      <xsl:with-param name="replace" select="$macroReplacePattern"/>
      <xsl:with-param name="by" select="cf:Genre"/>
    </xsl:call-template>
  </xsl:template>

  <!-- MAKRO-Kategorietext -->
  <xsl:template name="macro-Kategorietext">
    <xsl:value-of select="cf:Kategorietext"/>
  </xsl:template>

  <!-- MAKRO-Darsteller -->
  <xsl:template name="macro-Darsteller">
    <xsl:call-template name="string-replace">
      <xsl:with-param name="text" select="cf:Darsteller"/>
      <xsl:with-param name="replace" select="';'"/>
      <xsl:with-param name="by" select="', '"/>
    </xsl:call-template>
  </xsl:template>

  <!-- MAKRO-Beschreibung -->
  <xsl:template name="macro-Beschreibung">
    <!-- In der Beschreibung vom ClickFinder wird <br> (HTML-Code) als Zeilenumbruch verwendet. -->
    <!-- Dieser muss ersetzt werden, damit die Zeilenumbrüche richtig dargestellt werden. -->
    <!-- Die Ersetzung ist dabei Windows-konform, heißt es wird CRLF verwendet. -->
    <xsl:call-template name="string-replace">
      <xsl:with-param name="text" select="cf:Beschreibung" />
      <xsl:with-param name="replace" select="'&lt;br&gt;'" />
      <xsl:with-param name="by" select="' &#13;&#10;'" />
    </xsl:call-template>
  </xsl:template>

</xsl:stylesheet>